/*
 * Copyright 2016, China Mobile, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain
 * a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

#ifndef __TCMU_CONFIG_H
# define __TCMU_CONFIG_H

#include <stdbool.h>
#include <pthread.h>

#include "ccan/list/list.h"

struct tcmu_config {
	pthread_t thread_id;
	char path[PATH_MAX];

	bool is_dynamic;
	int log_level;
	int def_log_level;

	char log_dir[PATH_MAX];
	char def_log_dir[PATH_MAX];
};

/*
 * There are 6 logging levels supported in tcmu.conf:
 *    0: CRIT
 *    1: ERROR
 *    2: WARNING
 *    3: INFO
 *    4: DEBUG
 *    5: DEBUG SCSI CMD
 */
enum {
	TCMU_CONF_LOG_LEVEL_MIN = 0,
	TCMU_CONF_LOG_CRIT = 0,
	TCMU_CONF_LOG_ERROR = 1,
	TCMU_CONF_LOG_WARN,
	TCMU_CONF_LOG_INFO,
	TCMU_CONF_LOG_DEBUG,
	TCMU_CONF_LOG_DEBUG_SCSI_CMD,
	TCMU_CONF_LOG_LEVEL_MAX = TCMU_CONF_LOG_DEBUG_SCSI_CMD,
};

static const char *const log_level_lookup[] = {
	[TCMU_CONF_LOG_CRIT]	= "CRIT",
	[TCMU_CONF_LOG_ERROR]	= "ERROR",
	[TCMU_CONF_LOG_WARN]	= "WARNING",
	[TCMU_CONF_LOG_INFO]	= "INFO",
	[TCMU_CONF_LOG_DEBUG]	= "DEBUG",
	[TCMU_CONF_LOG_DEBUG_SCSI_CMD]	= "DEBUG SCSI CMD",
};

struct tcmu_config* tcmu_initiatize_config(void);
void tcmu_destroy_config(struct tcmu_config *cfg);
int tcmu_setup_config(struct tcmu_config *cfg);
#endif /* __TCMU_CONFIG_H */
